/***************************************************************************
 *   Copyright (C) 2008 by Robin Atwood   *
 *   robin@binro.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "krclistview.h"
#include <kicon.h>

KrcListView::KrcListView( QWidget *parent )
 : QTreeWidget(parent)
{
	setHeaderLabel( "Services" );
	sortItems( 0, Qt::AscendingOrder );
	setRootIsDecorated( false );
	setAllColumnsShowFocus( true );
	setAlternatingRowColors( true );
	setColumnCount( 1 );
	setUniformRowHeights( true );
	setSelectionMode( QAbstractItemView::ExtendedSelection );
	m_dragStart = false;
	
  KIconLoader il;
  m_minipx = new QPixmap( il.loadIcon( "service", KIconLoader::Small ) );
	setToolTip( i18n("Drag services to required runlevel") );
}


KrcListView::~KrcListView()
{
}

void KrcListView::mousePressEvent( QMouseEvent *me )
{
  m_dragStart = true;
  m_startPos = me->pos();
  QTreeWidget::mousePressEvent( me );
}

void KrcListView::mouseMoveEvent( QMouseEvent *me )
{
  if ( !m_dragStart ) return;
  
  int mindragdist = KGlobalSettings::dndEventDelay();

  if ( Qt::LeftButton && (me->pos() - m_startPos).manhattanLength() > mindragdist )
 	{
  	QList<QTreeWidgetItem*> items = selectedItems();
  	QString services;
  	
  	for ( int k = 0; k < items.count(); ++k )
			services += items[k]->text( 0 ) + ' ';
  	
    QDrag *qdrag = new QDrag( this );
		QMimeData *mimeData = new QMimeData;
		
		mimeData->setText( services );
    qdrag->setMimeData(mimeData);
    qdrag->setPixmap( *m_minipx );
		qdrag->setHotSpot( QPoint(-5, -5) );
    qdrag->exec( Qt::CopyAction|Qt::MoveAction, Qt::CopyAction );
    m_dragStart = false;
  }
}


#include "krclistview.moc"


