/***************************************************************************
 *   Copyright (C) 2008 by Robin Atwood   *
 *   robin@binro.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qwidget.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdir.h>
//Added by qt3to4:
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QDebug>

#include <kglobalsettings.h>
#include <kmessagebox.h>

#include "krcrlvlview.h"
#include "krclistitem.h"
#include "krcstat.h"

#include <errno.h>
#include <unistd.h>

extern int errno;

KrcRlvlView::KrcRlvlView( QWidget *parent, KrcStat *mainWin )
 : QTreeWidget( parent )
{ 
	QStringList labels;
	int nCols;
	
	labels <<  "Service" << "Status" << "Daemon" << "Runlevel";
	nCols = labels.count();
	if ( mainWin->m_base > 1 )
	{
		 labels << "Description";
		 ++nCols;
	}

  setHeaderLabels( labels );
	setColumnCount( nCols );
  setAllColumnsShowFocus( true );
  setRootIsDecorated( false );
	setContextMenuPolicy( Qt::CustomContextMenu );
	setSortingEnabled( true );
	setAlternatingRowColors( true );
	setUniformRowHeights( true );
	setAcceptDrops( true );


	m_services = &mainWin->m_services;
	m_runStatus = &mainWin->runStatus;
	m_mw = mainWin;
	m_allView = mainWin->m_all;
	m_textColors = &mainWin->textColors;
	
	//connect( this, SIGNAL( needRefresh() ), mainWin, SLOT( slotRefresh() ) );
	
	QString info = i18n(
		"<h3>Runlevel list</h3>"
		"This pane displays a list of services that will be started when this "
		"runlevel is activated. <br><br>To add more, simply drag a service from the service pane.");
		
	setWhatsThis( info );
}

KrcRlvlView::~KrcRlvlView()
{
}

void KrcRlvlView::load( const QString rlvl )
{
	QDir rlvlDir( RLVLDIR + rlvl, "", 0, QDir::System | QDir::Files );
	
	m_name = rlvl;
	m_assigned = rlvlDir.entryList();
	
	for ( QStringList::Iterator name = m_assigned.begin(); name != m_assigned.end(); ++name ) 
	{
		QString status, daemon, desc;
		KrcService *serv;
		
		m_mw->getStatus( *name, status, daemon, desc );
		serv = m_services->find( *name );
		if ( serv > 0 )
			serv->m_rlvls << rlvl;
		else
			status = "broken";
		new KrcListItem( this, *name, status, daemon, m_name, desc, m_textColors );
		new KrcListItem( m_allView, *name, status, daemon, m_name, desc, m_textColors );
	}

	sortItems( 0, Qt::AscendingOrder );
}

void KrcRlvlView::dragEnterEvent( QDragEnterEvent* event )
{
	if ( event->mimeData()->hasText() )
	{	
		event->acceptProposedAction();
	}	
	else
		event->ignore();
}

void KrcRlvlView::dragMoveEvent( QDragMoveEvent* event )
{
	event->acceptProposedAction();
}

void KrcRlvlView::dropEvent( QDropEvent* event )
{
  QString text, svc;
	
	if ( event->mimeData()->hasText() ) 
	{
		if ( event->source() == 0 ) return;           // not our application
		event->acceptProposedAction();
		
		text = event->mimeData()->text();
		for ( int k = 0;; ++k )
		{
			svc = text.section( ' ', k, k );
			if ( svc == "" ) break;
			
			if ( m_services->find( svc ) > 0 )
				createLink( svc );
		}

		//emit needRefresh();
  }
  else
		event->ignore();
}

bool KrcRlvlView::createLink( QString svc )
{
	QString oldpath = SERVDIR + svc, 
	        newpath = RLVLDIR + m_name + "/" + svc;
					
	if ( m_mw->m_view->m_term > 0 )
		m_mw->m_view->m_term->sendInput( "ln -s " + oldpath + " " + newpath  + "\n" );
	
// 	if ( symlink( oldpath.toLatin1(), newpath.toLatin1() ) < 0 )
// 	{
// 		KMessageBox::error( this, QString( i18n("Service assignment failed: ") ) + strerror( errno ) );
// 	 	return false;
// 	}
// 	else
	return true;       
}

#include "krcrlvlview.moc"



