/***************************************************************************
 *   Copyright (C) 2008 by Robin Atwood   *
 *   robin@binro.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KRCSTAT_H_
#define _KRCSTAT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kxmlguiwindow.h>
#include <kprocess.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <klineedit.h>
#include <knotification.h>
#include <knotifyconfigwidget.h>

#include "krcmainview.h"
#include "krcrlvlview.h"
#include "krcallview.h"
#include "krclistitem.h"
#include "krcservice.h"

#include <qlabel.h>
#include <qfilesystemwatcher.h>
//Added by qt3to4:
#include <QPixmap>

/**
 * This class serves as the main window for krcstat.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Robin Atwood <robin@binro.org>
 * @version 0.1
 */
class KrcStat : public KXmlGuiWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    KrcStat( QWidget* parent = 0 );

    /**
     * Default Destructor
     */
    virtual ~KrcStat();

		friend class KrcRlvlView;
		int m_base;
		
protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveOptions();

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readOptions();
    
    virtual bool queryExit();    

public slots:
    void slotRefresh();
		void slotUpdate();
		
private slots:
		void slotCmdAct( QAction* );
    void slotRemove();
		void slotNewRlvl();
    void slotRmRlvl();
    void slotEraseEdit();
    void slotEditRlvl( const QString & );
		void slotRMBItem( const QPoint &point );
		void slotRowChanged();
		void slotDirChanged( const QString & );
		void slotFailedNote();
		void slotShellExit();
		void optionsPreferences();
		void configureNotifications();
		
signals:
		void needRefresh();

private:
    void setupAccel();
    void setupActions();
		KAction* newAction(const QString &, const KIcon &, const QKeySequence &, QObject *, const QString & );

		void loadServices( const bool reload = false );
		void loadRunLevels( const bool reload = false );
		void resizeColumns ( QTreeWidget* );
		void updateServices();
		void updateDaemons( const QString & );
		void setStats();
		void getStatus( const QString &, QString &, QString &, QString & );
		QStringList sysCommand( const QString & );    
		void issueCommand( const QString & );    
		bool pidExists( const QString & );
    QString getDescriptor(const QString& svc);
    QString getDaemon(const QString& svc);
    QString getOptions(const QString& svc);
		QString extractCommands( const QString &variable, const QString &script);
    QString servInfo(const KrcService& service);
		//void execKonsole( const QString, const QString );
    
    KrcMainView *m_view;
    KrcRlvlView *m_rlvl;
    KrcAllView *m_all;

		KrcStatus runStatus;
    KrcColors textColors;
    KrcServiceList m_services;
    QStringList m_runlevels, m_assigned;
    QString m_softlevel;
    KAction *fileStart;
    KAction *fileStop;
    KAction *fileRestart;
    KAction *fileReload;
    KAction *filePauseResume;
    KAction *fileRefresh;
    KAction *fileZap;
    KAction *fileHelp;
    KAction *fileRemove;
    KAction *rlvlAdd;
    KAction *rlvlEntry;
    KAction *rlvlDelete;
		QActionGroup *m_actGrp, *m_actSubGrp;
		KLineEdit *m_rlvlEdit;
    QMenu *m_popup, *m_subpopup;
    QTimer  *m_timer;
		QLabel  *m_SBtext1, *m_SBtext2, *m_SBtext3;
    QPixmap *m_minipx;
    QPixmap *m_px;
    QPixmap *m_usedpx;
    QPixmap *m_unusedpx;
    int m_updtime;
    QString m_saveName;
    QString m_saveRlvl;
    QString m_saveStatus;
    bool m_firstView;
    uint m_curPage;
    KSharedConfigPtr m_config;
		KConfigGroup *m_colorGroup;
		QStringList StatDir;
		const QString rlvlTB;
		QList<QAction *> m_actList1, m_actList2;
		QFileSystemWatcher *m_FSW;
		KNotification *m_note;
		bool m_updating;
};

#endif // _KRCSTAT_H_

